<?php


namespace NL\NlMenubuilder\Service;


use NL\NlMenubuilder\Domain\Model\MenuInterface;
use NL\NlMenubuilder\Domain\Model\MenuItem;
use NL\NlMenubuilder\Domain\Model\MenuItemPage;
use TYPO3\CMS\Core\SingletonInterface;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Utility\RootlineUtility;
use TYPO3\CMS\Frontend\Controller\TypoScriptFrontendController;

class MenuSecurityService implements SingletonInterface
{
    /**
     * @param MenuInterface $menu
     * @throws \TYPO3\CMS\Core\Context\Exception\AspectPropertyNotFoundException
     */
    public function applySecurityCheck(MenuInterface $menu): void
    {
        $this->removeInaccessible($menu);

        $this->removeEmpty($menu);
    }

    /**
     * @param MenuInterface $menu
     * @throws \TYPO3\CMS\Core\Context\Exception\AspectPropertyNotFoundException
     */
    protected function removeInaccessible(MenuInterface $menu): void
    {
        foreach ($menu->getPageItems() as $pageItem) {
            if (false === $this->isAccessible($pageItem)) {
                $menu->removeItem($pageItem);
            }
        }

        foreach ($menu->getSubmenuItems() as $submenuItem) {
            $this->removeInaccessible($submenuItem);
        }
    }

    /**
     * @param MenuInterface $menu
     */
    protected function removeEmpty(MenuInterface $menu): void
    {
        /** @var MenuItem $submenuItem */
        foreach ($menu->getSubmenuItems() as $submenuItem) {
            $this->removeEmpty($submenuItem);

            if (0 === $submenuItem->getItems()->count()) {
                $menu->removeItem($submenuItem);
            }
        }
    }

    /**
     * @param MenuItemPage $itemPage
     * @return bool
     * @throws \TYPO3\CMS\Core\Context\Exception\AspectPropertyNotFoundException
     */
    protected function isAccessible(MenuItemPage $itemPage): bool
    {
        if (null === $itemPage->getPage()) {
            return false;
        }

        /** @var RootlineUtility $rootline */
        $rootline = GeneralUtility::makeInstance(RootlineUtility::class, $itemPage->getPage()->getUid());

        foreach ($rootline->get() as $item) {
            if (!$this->getTypoScriptFrontendController()->checkPagerecordForIncludeSection($item)) {
                return false;
            }
        }

        return true;
    }

    /**
     * @return TypoScriptFrontendController
     */
    protected function getTypoScriptFrontendController(): TypoScriptFrontendController
    {
        return $GLOBALS['TSFE'];
    }
}
